/*
 * NPACLogo.java
 * Author: Alvin Leung
 *
 * NPACLogo - applet and main thread to do the image loop and initialize 
 *    image generation thread.  This thread is called AnimationEngine
 *    and cycles through the images to display them.
 *
 * Logo - the thread generates the images for animation sequence and 
 *    stores them in a vector called images.
 *
 * Parallelogram - helper object to draw the panels.
 * Copyright Notices
 *   (C) 1995 - 1996      Northeast Parallel Architectures Center at
 *                        Syracuse University
 */

import java.applet.Applet;
import java.awt.Event;
import java.awt.Component;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.Color;
import java.util.Vector;

import Parallelogram;  //  class to draw one panel parallelogram

public class NPACLogo extends Applet implements Runnable {
   // Animation Specific data
   static final int AnimationLength = 7;
   // Animation Specific defaults
   int    width = 356;
   int    height = 256;
   long   delay = 400;

// vector to store images - accessed by both threads
   Vector images = new Vector(AnimationLength);
   int    index = 0;	//  next image to show
   int    factor = 1;   //  increment to index
   Thread AnimationEngine = null;    // applet thread
   Logo ImageEngine = null;          // image generation thread
   boolean userPause = false;

/* applet classes have methods init, start, and stop for initialization
of applet, and starting and stopping of applet window */

   public void init() {
      int   i;
      String Parameter;

      Parameter = getParameter("width");
      if (Parameter != null)
         width = Integer.parseInt(Parameter);

      Parameter = getParameter("height");
      if (Parameter != null)
          height = Integer.parseInt(Parameter);

      Parameter = getParameter("delay");
      if (Parameter != null)
         delay = Long.parseLong(Parameter);

    // creates and starts image generation thread
      ImageEngine = new Logo(AnimationLength, width, height, images, this);
      if (ImageEngine != null)
      {
         ImageEngine.setPriority(Thread.MAX_PRIORITY);
         ImageEngine.start();
      }
   }

   public void start() {
      if (AnimationEngine == null) {
         AnimationEngine = new Thread(this);
         AnimationEngine.setPriority(Thread.NORM_PRIORITY);
         AnimationEngine.start();
      }
   }

   public void stop() {
      if (AnimationEngine != null && AnimationEngine.isAlive()) {
         AnimationEngine.stop();
      }
      AnimationEngine = null;
      if (ImageEngine != null && ImageEngine.isAlive()) {
         ImageEngine.stop();
      }
      ImageEngine = null;
   }

/* a class which implements Runnable must have a run method for its 
     thread */

   public void run() {
      while (true)
      {
         try {
            Thread.sleep((long) delay);
         } catch (InterruptedException e) {
            System.out.println("Interrupted!");
            break;
         }
         repaint();
      }
   }

/* update method is called by repaint to initiate screen redrawing.
   The keyword synchronized means that this method will run uninterrupted
   by any other thread.  */

   public synchronized void update(Graphics g) {
      g.drawImage((Image) images.elementAt(index), 0, 0, this);
      index += factor;
      if (index < images.size())
      {
         if (index < 0)
         {
            index += 2;
            factor = 1;
         }
      }
      else
      {
         if (index >= AnimationLength)
         {
            index -= 2;
            factor = -1;
         }
         else
            index -= factor;
      }
   }

/* paint method called the first time the applet runs to draw screen */

   public synchronized void paint(Graphics g) {
      g.drawImage((Image) images.elementAt(index), 0, 0, this);
   }

/* an applet method to handle event interrupts  */

   public boolean handleEvent(Event evt) {
      if (evt.id == Event.MOUSE_DOWN) {
         if (AnimationEngine != null && AnimationEngine.isAlive()) {
            if (userPause) {
               AnimationEngine.resume();
            } else {
               AnimationEngine.suspend();
               index = images.size() - 1;
               getGraphics().drawImage((Image) images.lastElement(), 0, 0,
                  this);
            }
            userPause = !userPause;
         } else {
            userPause = false;
            AnimationEngine = new Thread(this);
            AnimationEngine.start();
         }
         return true;
       } else {
           return super.handleEvent(evt);
       }
   }
}

class Logo extends Thread {
   int AnimationLength;
   Vector images;
   int width;
   int height;
   Component Observer = null;

/* class constructor */

   public Logo(int l, int w, int h, Vector ims, Component O) {
      AnimationLength = l;
      width = w;
      height = h;
      images = ims;
      Observer = O;

      int BackGroundHeight = 256;
      Image BackGround = Observer.createImage(width, BackGroundHeight);
      Graphics BackGroundGC = BackGround.getGraphics();

     // draws background waterfall
      for (int i = 0; i < BackGroundHeight; i++)
      {
         BackGroundGC.setColor(new Color(i, i, i));
         BackGroundGC.drawLine(0, i, width - 1, i);
      }

      Image image = Observer.createImage(width, height);
      Graphics imageGC = image.getGraphics();
      imageGC.drawImage(BackGround, 0, 0, width, height, Observer);
      images.addElement(image); 
   }

/* run method of this thread has loop to construct images with increasing
   number of parallelograms */

   public void run() {
      int x;
      int y;
      Image    image;
      Graphics imageGC;
      Parallelogram penel = new Parallelogram(width, height);

      while (images.size() < AnimationLength)
      {
         image = Observer.createImage(width, height);
         imageGC = image.getGraphics();

         switch (images.size())
         {
            case 1:
               x = (int) (width * (6.3 / 8.35));
               y = (int) (height * (2.5 / 5.9));
               break;

            case 2:
               x = (int) (width * (2.4 / 8.35));
               y = (int) (height * (1.95 / 5.9));
               break;

            case 3:
               x = (int) (width * (3.3 / 8.35));
               y = (int) (height * (0.35 / 5.9));
               break;

            case 4:
               x = (int) (width * (3.35 / 8.35));
               y = (int) (height * (2.7 / 5.9));
               break;

            case 5:
               x = (int) (width * (4.4 / 8.35));
               y = (int) (height * (1.35 / 5.9));
               break;

            case 6:
               x = (int) (width * (0.65 / 8.35));
               y = (int) (height * (0.55 / 5.9));
               break;

            default:
               x = 0;
               y = 0;
         }
         imageGC.drawImage((Image) images.lastElement(), 0, 0, width, height,
            Observer);
         penel.fill(imageGC, x, y);
         images.addElement(image);
      }

      return;
   }
}

